import React, { useState, useContext, useEffect } from "react";
import { useCookies } from "react-cookie";
import { useNavigate, useParams } from 'react-router-dom';
import { answersData } from "../context";
import classes from "../assets/styles/answersForm.module.scss";
import { responseDataToListBlock, dateTimeParse } from "../hooks/sundry/parseListBlock";
import { listFormBlockApi } from "../hooks/api/formApi";
import { listUsersApi } from "../hooks/api/adminApi";
import { removeAnswerFormApi, getAnswersApi } from "../hooks/api/answersApi.js";
import Loading from "../components/Loading.jsx";
import CheckModal from "../components/CheckModal.jsx";

const AnswersForm = () => {
    const { formId } = useParams();
    const {answersList, setAnswersList} = useContext(answersData);
    const [data, setData] = useState(false);
    const [cookies, _, __] = useCookies(["user"]);

    useEffect(() => {
        async function getForm() {
            const result = []
            const responseBlocks = await listFormBlockApi(cookies.token, formId);
            const responseAnswers = await getAnswersApi(cookies.token, formId);
            const listUsers = await listUsersApi(cookies.token);

            if (responseBlocks.status === 200 && responseBlocks.data) {
                const listBlocks = responseDataToListBlock(responseBlocks.data);
                
                if (responseAnswers.data) {
                    for (let item of responseAnswers.data) {
                        const blocks = {
                            id: item.id,
                            user: listUsers.data.find(user => user.id === item.user_id).login,
                            date: dateTimeParse(item.date),
                            block: []
                        }

                        if (item.data) {
                            for (let i = 0; i < item.data.length; i++) {
                                blocks.block.push({
                                    answers: {id: item.data[i].find(obj => obj.Key === "id").Value, answer: item.data[i].find(obj => obj.Key === "answer").Value}, 
                                    question: listBlocks[i]
                                })
                            }
                        }
                        else {
                            for (let i = 0; i < listBlocks.length; i++) {
                                blocks.block.push({answers: [], question: listBlocks[i]})
                            }
                        }

                        result.push(blocks)
                    }
                }

                setData(result)
            }
            else {
                console.log(responseBlocks)
            }
        };

        getForm()
    }, []);

    async function removeAnswerForm(answerId) {
        const response = await removeAnswerFormApi(cookies.token, formId, answerId)

        if (response.status === 200) {
            setData([...data].filter(item => item.id !== answerId))
        }
        else {
            console.log(response)
        }
    }

    return (
        <div className={classes.main}>
            <div className={classes.wrapper}>
                <div className={classes.answers}>
                    <div className={classes.answers__wrapper}>
                        <div className={classes.answers__wrapper__header}>
                            <h3>Ответы</h3>
                        </div>
                        <div className={classes.answers__wrapper__body}>
                            <div className={classes.answers__wrapper__body__column}>
                                <div className={classes.answers__wrapper__body__column__wrapper}>
                                    <div>Логин</div>
                                    <div>Дата</div>
                                </div>                              
                            </div>

                            {data ? 
                                data.map((item, dataIndex) => 
                                    <div className={classes.answers__wrapper__body__item} key={dataIndex}>
                                        <div className={classes.answers__wrapper__body__item__wrapper}>
                                            <div className={classes.answers__wrapper__body__item__wrapper__user} data-bs-toggle={"modal"} data-bs-target={`#answersModal${dataIndex}`}>
                                                {item.user}
                                            </div>
                                            <div className={classes.answers__wrapper__body__item__wrapper__date}>
                                                {item.date}
                                            </div>
                                            <i class="fa-solid fa-trash" data-bs-toggle="modal" data-bs-target={`#checkModal${item.id}`}></i>
                                        </div>   

                                        <CheckModal
                                            postfix={item.id}
                                            message={`Вы хотетите удалить ответы пользователя <${item.user}>?`} 
                                            action={{
                                                execute: () => removeAnswerForm(item.id), 
                                                cancel: () => {}
                                            }}
                                        />                                     

                                        <div class="modal fade myModal" className={classes.myModal} id={`answersModal${dataIndex}`} tabIndex="-1" aria-labelledby="exampleModalLabel" data-bs-backdrop="static" aria-hidden="true">
                                            <div class="modal-dialog myModal__dialog" className={classes.myModal__dialog}>
                                                <div class="modal-content" className={classes.myModal__dialog__content}>
                                                    <div class="modal-body" className={classes.myModal__dialog__content__body}>
                                                        {item.block.map((block, blockIndex) => 
                                                        <div className={classes.myModal__item} key={blockIndex}>
                                                            <div className={classes.myModal__item__question}>
                                                                <p className={classes.myModal__item__question__text}>{block.question.question}</p>
                                                                <p className={classes.myModal__item__question__comment}>{block.question.comment}</p>
                                                            </div>
                                                            <div className={classes.myModal__item__answer}>
                                                                <p className={classes.myModal__item__question__text}>Ответ: {
                                                                    Array.isArray(block.answers.answer) ? 
                                                                    block.answers.answer.map((answer, indexAnswer) => <span key={indexAnswer}>{answer}; </span>) :
                                                                    block.answers.answer
                                                                }</p>
                                                            </div>
                                                        </div>)}
                                                    </div>
                                                    <div class="modal-footer myModal__dialog__content__footer" className={classes.myModal__dialog__content__footer}>
                                                        <button type="button" class="btn btn-light" data-bs-dismiss="modal">Отмена</button>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                )
                            : 
                            <Loading/>}
                        </div>                        
                    </div>
                </div>
            </div>
        </div>
    )
}

export default AnswersForm;